/*******************************************************************
Programm: at90can128

Beschreibung: CAN Treiber fr den AT90can128


Anleitung:

Lizenz: non Plublic no Freeware
Copyright: Markus Rath

*******************************************************************/
#include <avr/io.h>           
#include "at90can128.h"


//Globale Variablen
typedef struct
	{	
		uint16_t id;			// MSG ID 11 Bit                  
		uint16_t msk;        	// Maske fr ID 11 Bit          
		uint8_t length;   		// DLC Lnge
		uint8_t data[8];  		// Nachrichten Text 8 Byte
	}CAN_messageType; // fr CAN 2.0 A nachrichten 11 Bit

CAN_messageType recMsg;




uint8_t getmob( uint32_t bitmask) 
{
 uint8_t mob;

 if( bitmask==0) return NOMOB;

 for( mob=0; (bitmask & 0x01)==0; bitmask >>= 1, ++mob);

 //security check
 if ( mob > 14 ) return NOMOB;
 	else return mob;
}     



void atcan90can128_init()
{

uint8_t mob;


  CANGCON |= (1<<SWRES);
  
  CANGCON = 0x00;
  CANGIT  = 0x00;
  CANSIT2 = 0x00;	
  CANSIT1 = 0x00;


/*Allo Mobs auf 0 setzten*/

  for (mob = 0; mob < NO_MOBS; mob++)
  {
     CANPAGE  = (mob << 4);
	 CANIDT1 = 0x00;		//reset ID-Tag
	 CANIDT2 = 0x00;
	 CANIDT3 = 0x00;
	 CANIDT4 = 0x00;

	 CANIDM1 = 0x00;		//reset ID-Mask
	 CANIDM2 = 0x00;
	 CANIDM3 = 0x00;
	 CANIDM4 = 0x00;

     CANSTMOB = 0x00;		//reset MOb status
     CANCDMOB = 0x00; 		//disable MOb
  }

	
	  //Enable CAN interrupts globally, set enable all interrupts except CANtimer overrun interrupt
  CANGIE = 0xfe;	//(1<<ENIT) | (1<<ENRX) | (1<<ENTX) |(1<<ENERR);//
  
  //Interrups fr MOBs freischalten
  CANIE2 = (1<<IEMOB1); //(1<<IEMOB0) |  (1<<IEMOB1) |   (1<<IEMOB3); 		
  CANIE1 = 0;

  
  /* auf 100 MBIT bei 16Mhz stellen */
  CANBT1 = 0x12;
  CANBT2 = 0x0C;
  CANBT3 = 0x37;
   
  CANTCON = 0x00; 	// Clock teiler fr Timestamps????
       
  CANGCON |= (1<<ENASTB);	// Can einschalten
  	
  // EnableFlag abwarten
  while (!(CANGSTA & (1<<ENFG))); 

}

void can_tx (CAN_messageType msg)
{
uint8_t i, mob;	

	//Mob Sender im TEST ist es die MOB 3
	mob=3;

	//enable MOb number mob, auto increment index, start with index = 0
	CANPAGE = (mob<<4);
	
	//set IDE bit, length = 8
	CANCDMOB = (0<<IDE) | (msg.length);		//ide 1 = extendet und ide 0 normal	
	
	// write 11 Bit identifier to MOb-ID-Tag
	
	msg.id <<= 5;								
	CANIDT2 = (unsigned char) (msg.id&0xE0);
	CANIDT1 = (unsigned char) (msg.id>>8);

	//msg daten ins mob schreiben
	for (i=0; i<8; i++)							
		CANMSG = msg.data [i];
	
	//tybsy abwarten
	while (CANGSTA & (1<<TXBSY));
	
	//nachricht senden
	CANCDMOB |= (1<<CONMOB0);

}
void can_txa (CAN_messageType msg)
{
uint8_t i, mob;	

	//Mob Sender im TEST ist es die MOB 3
	mob=4;

	//enable MOb number mob, auto increment index, start with index = 0
	CANPAGE = (mob<<4);
	
	//set IDE bit, length = 8
	CANCDMOB = 0;
	CANCDMOB = (0<<IDE) | (msg.length);		//ide 1 = extendet und ide 0 normal	
	
	// write 11 Bit identifier to MOb-ID-Tag
	
	msg.id <<= 5;								
	CANIDT2 = (unsigned char) (msg.id&0xE0);
	CANIDT1 = (unsigned char) (msg.id>>8);

	//msg daten ins mob schreiben
	for (i=0; i<8; i++)							
		CANMSG = msg.data [i];
	
	//tybsy abwarten
	while (CANGSTA & (1<<TXBSY));
	
	//nachricht senden
	CANCDMOB |= (1<<CONMOB0);
	//clear_mob();

}
void can_txb (CAN_messageType msg)
{
uint8_t i, mob;	

	//Mob Sender im TEST ist es die MOB 3
	mob=5;

	CANPAGE = (mob<<4);
	
	CANCDMOB = (0<<IDE) | (msg.length);		//ide 1 = extendet und ide 0 normal	
	
	// write 11 Bit identifier to MOb-ID-Tag
	
	msg.id <<= 5;								
	CANIDT2 = (unsigned char) (msg.id&0xE0);
	CANIDT1 = (unsigned char) (msg.id>>8);

	//msg daten ins mob schreiben
	for (i=0; i<8; i++)							
		CANMSG = msg.data [i];
	
	//tybsy abwarten
	while (CANGSTA & (1<<TXBSY));
	
	//nachricht senden
	CANCDMOB |= (1<<CONMOB0);

}

void can_rx_config ()
{	
uint8_t mob;

// in dieses MOB werden daten Entfangen (im test noch ohne filter ohne nix)
	mob = 1;

	CANPAGE = (mob<<4);				// um 4 bits nach lins routieren	
	
	CANIDT4 = 0;		
	CANIDT3 = 0;	
	CANIDT2 = 0;	
	CANIDT1 = 0;	

	// define an ID mask for the MOb. 
	
	CANIDM4 = 0;
	CANIDM3 = 0;
	CANIDM2 = 0;
	CANIDM1 = 0;

	// empfang freischalten fr 11 bit ids
	CANCDMOB = (1<<CONMOB1);
// --------------------------------------------------------------------------------------------

}

SIGNAL(SIG_CAN_INTERRUPT1)
{
//CAN_messageType recMsg; this is a global variable
uint8_t save_canpage = CANPAGE;		// aktuelle CANPAGE
uint8_t mob;
uint8_t i;


	// rausfinden in was fr ein mob die empfange nachricht ist
	mob = getmob(CANSIT2 | (CANSIT1 << 8));
		if( mob == NOMOB )   return;

	//select concerned MOb page
	CANPAGE = mob << 4;	


	if (CANSTMOB & (1<<TXOK))
		{
		 	
		 CANSTMOB &= ~(1<<TXOK);	// reset INT reason
				
		}
	else if (CANSTMOB & (1<<RXOK))
		{
			 //get data of selected MOb
			 for (i=0; i<8; i++)		
		 		 recMsg.data[i] = CANMSG;
	
			 //id auslesen buggy???
			 recMsg.id = 0;
			 recMsg.id |= ((uint16_t) CANIDT1<<8);
			 recMsg.id |= (CANIDT2&0xE0);
			 recMsg.id >>= 5;

             //lnge muss auch noch ausgelesen werden....

		 CANSTMOB &= ~(1<<RXOK);	// interrupt reseten 	
		 CANCDMOB = (1<<CONMOB1); 	// wieder auf empfang schalten


		}
	else 
		{

		}
		
 
	
	CANPAGE = save_canpage;   	// restore CANPAGE

	can_tx(recMsg);  //echo der empfangen nachricht  debug und timingtests


}
